@@ -32,17 +32,12 @@
 #include <stdarg.h>
 #include <limits.h>
 #include <string.h>
-#include <ctype.h>
 #include <errno.h>
-#include <time.h>
 #include <getopt.h>
 extern char *optarg;
 extern int optind;
-#include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
 #include <signal.h>
-#include <fcntl.h>
 #ifdef ENABLE_NLS
 # include <locale.h>
 #endif
@@ -59,7 +54,7 @@ extern int optind;
 #include "pop3.h"
 #include "password.h"
 #ifdef HAVE_TLS
-#include "tls.h"
+#include "mtls.h"
 #endif /* HAVE_TLS */
 #include "uidls.h"
 
@@ -148,8 +143,8 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
     pop3_session_t *session;
     char server_greeting[POP3_BUFSIZE - 4];
 #ifdef HAVE_TLS
-    tls_cert_info_t *tci = NULL;
-    char *tls_parameter_description = NULL;
+    mtls_cert_info_t *tci = NULL;
+    char *mtls_parameter_description = NULL;
 #endif /* HAVE_TLS */
     const char *server_canonical_name;
     const char *server_address;
@@ -176,7 +171,7 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
 #ifdef HAVE_TLS
     if (acc->tls)
     {
-        tci = tls_cert_info_new();
+        tci = mtls_cert_info_new();
         if ((e = pop3_tls_init(session,
                         acc->tls_key_file, acc->tls_cert_file, acc->password,
                         acc->tls_trust_file, acc->tls_crl_file,
@@ -189,7 +184,7 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
                         errstr)) != TLS_EOK)
         {
             pop3_session_free(session);
-            e = tls_exitcode(e);
+            e = mtls_exitcode(e);
             goto error_exit;
         }
     }
@@ -200,10 +195,10 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
     if (acc->tls && acc->tls_nostarttls)
     {
         if ((e = pop3_tls(session, tci,
-                        &tls_parameter_description, errstr)) != TLS_EOK)
+                        &mtls_parameter_description, errstr)) != TLS_EOK)
         {
             mpop_endsession(session, 0);
-            e = tls_exitcode(e);
+            e = mtls_exitcode(e);
             goto error_exit;
         }
     }
@@ -246,10 +241,10 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
             goto error_exit;
         }
         if ((e = pop3_tls(session, tci,
-                        &tls_parameter_description, errstr)) != TLS_EOK)
+                        &mtls_parameter_description, errstr)) != TLS_EOK)
         {
             mpop_endsession(session, 0);
-            e = tls_exitcode(e);
+            e = mtls_exitcode(e);
             goto error_exit;
         }
         /* get capabilities again */
@@ -326,7 +321,7 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
 #ifdef HAVE_TLS
     if (acc->tls)
     {
-        tls_print_info(tls_parameter_description, tci);
+        mtls_print_info(mtls_parameter_description, tci);
     }
 #endif /* not HAVE_TLS */
     printf(_("POP3 capabilities:\n"));
@@ -360,17 +355,20 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
     {
         printf("    LOGIN-DELAY %ld:\n        ",
                 session->cap.login_delay);
-        printf(_("minimum time between logins is %ld seconds"),
-                session->cap.login_delay);
         if (session->cap.login_delay > 60 * 60)
         {
-            printf(_(" = %.2f hours"),
-                    (float)session->cap.login_delay / (60.0 * 60.0));
+            printf(_("minimum time between logins is %ld seconds = %.2f hours"),
+                    session->cap.login_delay, session->cap.login_delay / (60.0f * 60.0f));
         }
         else if (session->cap.login_delay > 60)
         {
-            printf(_(" = %.2f minutes"),
-                    (float)session->cap.login_delay / 60.0);
+            printf(_("minimum time between logins is %ld seconds = %.2f minutes"),
+                    session->cap.login_delay, session->cap.login_delay / 60.0f);
+        }
+        else
+        {
+            printf(_("minimum time between logins is %ld seconds"),
+                    session->cap.login_delay);
         }
         printf("\n");
     }
@@ -451,6 +449,10 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
     {
         printf("OAUTHBEARER ");
     }
+    if (session->cap.flags & POP3_CAP_AUTH_XOAUTH2)
+    {
+        printf("XOAUTH2 ");
+    }
     printf("\n");
     if (session->cap.flags & POP3_CAP_RESP_CODES)
     {
@@ -493,8 +495,8 @@ error_exit:
 #ifdef HAVE_TLS
     if (tci)
     {
-        tls_cert_info_free(tci);
-        free(tls_parameter_description);
+        mtls_cert_info_free(tci);
+        free(mtls_parameter_description);
     }
 #endif /* HAVE_TLS */
     return e;
@@ -635,8 +637,8 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 {
     pop3_session_t *session;
 #ifdef HAVE_TLS
-    tls_cert_info_t *tci = NULL;
-    char *tls_parameter_description = NULL;
+    mtls_cert_info_t *tci = NULL;
+    char *mtls_parameter_description = NULL;
 #endif /* HAVE_TLS */
     int e;
     long i, j;
@@ -685,7 +687,7 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
                         errstr)) != TLS_EOK)
         {
             pop3_session_free(session);
-            return tls_exitcode(e);
+            return mtls_exitcode(e);
         }
     }
 #endif /* HAVE_TLS */
@@ -696,24 +698,24 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     {
         if (debug)
         {
-            tci = tls_cert_info_new();
+            tci = mtls_cert_info_new();
         }
         if ((e = pop3_tls(session, tci,
-                        &tls_parameter_description, errstr)) != POP3_EOK)
+                        &mtls_parameter_description, errstr)) != POP3_EOK)
         {
             if (debug)
             {
-                tls_cert_info_free(tci);
-                free(tls_parameter_description);
+                mtls_cert_info_free(tci);
+                free(mtls_parameter_description);
             }
             mpop_endsession(session, 0);
-            return tls_exitcode(e);
+            return mtls_exitcode(e);
         }
         if (debug)
         {
-            tls_print_info(tls_parameter_description, tci);
-            tls_cert_info_free(tci);
-            free(tls_parameter_description);
+            mtls_print_info(mtls_parameter_description, tci);
+            mtls_cert_info_free(tci);
+            free(mtls_parameter_description);
         }
     }
 #endif /* HAVE_TLS */
@@ -751,24 +753,24 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
         }
         if (debug)
         {
-            tci = tls_cert_info_new();
+            tci = mtls_cert_info_new();
         }
         if ((e = pop3_tls(session, tci,
-                        &tls_parameter_description, errstr)) != TLS_EOK)
+                        &mtls_parameter_description, errstr)) != TLS_EOK)
         {
             if (debug)
             {
-                tls_cert_info_free(tci);
-                free(tls_parameter_description);
+                mtls_cert_info_free(tci);
+                free(mtls_parameter_description);
             }
             mpop_endsession(session, 0);
-            return tls_exitcode(e);
+            return mtls_exitcode(e);
         }
         if (debug)
         {
-            tls_print_info(tls_parameter_description, tci);
-            tls_cert_info_free(tci);
-            free(tls_parameter_description);
+            mtls_print_info(mtls_parameter_description, tci);
+            mtls_cert_info_free(tci);
+            free(mtls_parameter_description);
         }
         /* get capabilities again */
         if ((session->cap.flags & POP3_CAP_CAPA)
@@ -2161,6 +2163,8 @@ int main(int argc, char *argv[])
                 "GnuTLS"
 #elif defined (HAVE_LIBSSL)
                 "OpenSSL"
+#elif defined (HAVE_LIBTLS)
+                "libtls"
 #else
                 _("none")
 #endif
@@ -2169,7 +2173,7 @@ int main(int argc, char *argv[])
         printf(_("Authentication library: %s\n"
                     "Supported authentication methods:\n"),
 #ifdef HAVE_LIBGSASL
-                _("GNU SASL; user, apop, oauthbearer: built-in")
+                _("GNU SASL; user, apop, oauthbearer, xoauth2: built-in")
 #else
                 _("built-in")
 #endif /* HAVE_LIBGSASL */
@@ -2218,6 +2222,10 @@ int main(int argc, char *argv[])
         {
             printf("oauthbearer ");
         }
+        if (pop3_client_supports_authmech("XOAUTH2"))
+        {
+            printf("xoauth2 ");
+        }
         printf("\n");
         /* Internationalized Domain Names support */
         printf(_("IDN support: "));
@@ -2267,10 +2275,11 @@ int main(int argc, char *argv[])
     {
         printf(_("Usage:\n\n"));
         printf(_("Mail retrieval mode (default):\n"
-                    "  %s [option...] [--] account...\n"
+                    "  %s [option...] account...\n"
+                    "  %s [option...] --all-accounts\n"
                     "  %s --host=host [option...]\n"
-                    "  Read mails from one ore more POP3 accounts and deliver "
-                    "them.\n"), prgname, prgname);
+                    "  Read mails from one or more POP3 accounts and deliver "
+                    "them.\n"), prgname, prgname, prgname);
         printf(_("Configuration mode:\n"
                     "  %s --configure=mailaddress\n"
                     "  Generate and print configuration for address.\n"), prgname);
@@ -2656,7 +2665,7 @@ int main(int argc, char *argv[])
 #ifdef HAVE_TLS
             if (!tls_lib_initialized)
             {
-                if (tls_lib_init(&errstr) != TLS_EOK)
+                if (mtls_lib_init(&errstr) != TLS_EOK)
                 {
                     print_error(_("cannot initialize TLS library: %s"),
                             sanitize_string(errstr));
@@ -2789,7 +2798,7 @@ exit:
 #ifdef HAVE_TLS
     if (tls_lib_initialized)
     {
-        tls_lib_deinit();
+        mtls_lib_deinit();
     }
 #endif /* HAVE_TLS */
     if (net_lib_initialized)
