@@ -36,7 +36,6 @@
 #include <ctype.h>
 #include <errno.h>
 #include <signal.h>
-#include <unistd.h>
 
 #ifdef HAVE_LIBGSASL
 # include <gsasl.h>
@@ -54,7 +53,7 @@
 #include "tools.h"
 #include "stream.h"
 #ifdef HAVE_TLS
-# include "tls.h"
+# include "mtls.h"
 #endif /* HAVE_TLS */
 #include "pop3.h"
 #include "uidls.h"
@@ -140,7 +139,7 @@ pop3_session_t *pop3_session_new(int pipelining,
     session->server_address = NULL;
     readbuf_init(&(session->readbuf));
 #ifdef HAVE_TLS
-    tls_clear(&session->tls);
+    mtls_clear(&session->mtls);
 #endif /* HAVE_TLS */
     session->cap.flags = 0;
     session->pipelining = pipelining;
@@ -244,9 +243,9 @@ int pop3_gets(pop3_session_t *session, size_t *len, char **errstr)
     int e = 0;
 
 #ifdef HAVE_TLS
-    if (tls_is_active(&session->tls))
+    if (mtls_is_active(&session->mtls))
     {
-        e = (tls_gets(&session->tls, &(session->readbuf),
+        e = (mtls_gets(&session->mtls, &(session->readbuf),
                     session->buffer, POP3_BUFSIZE, len, errstr) != TLS_EOK);
     }
     else
@@ -389,9 +388,9 @@ int pop3_send_cmd(pop3_session_t *session, char **errstr,
     line[count++] = '\n';
     line[count] = '\0';
 #ifdef HAVE_TLS
-    if (tls_is_active(&session->tls))
+    if (mtls_is_active(&session->mtls))
     {
-        e = (tls_puts(&session->tls, line, (size_t)count, errstr) != TLS_EOK);
+        e = (mtls_puts(&session->mtls, line, (size_t)count, errstr) != TLS_EOK);
     }
     else
     {
@@ -868,6 +867,10 @@ int pop3_capa(pop3_session_t *session, char **errstr)
                 {
                     session->cap.flags |= POP3_CAP_AUTH_OAUTHBEARER;
                 }
+                if (strstr(session->buffer + 5, "XOAUTH2"))
+                {
+                    session->cap.flags |= POP3_CAP_AUTH_XOAUTH2;
+                }
             }
             else if (strncmp(session->buffer, "IMPLEMENTATION", 14) == 0)
             {
@@ -938,7 +941,7 @@ int pop3_tls_init(pop3_session_t *session,
         int no_certcheck,
         char **errstr)
 {
-    return tls_init(&session->tls, tls_key_file, tls_cert_file, pin,
+    return mtls_init(&session->mtls, tls_key_file, tls_cert_file, pin,
             tls_trust_file, tls_crl_file,
             tls_sha256_fingerprint, tls_sha1_fingerprint, tls_md5_fingerprint,
             min_dh_prime_bits, priorities, hostname, no_certcheck, errstr);
@@ -990,9 +993,9 @@ int pop3_tls_stls(pop3_session_t *session, char **errmsg, char **errstr)
 
 #ifdef HAVE_TLS
 int pop3_tls(pop3_session_t *session,
-        tls_cert_info_t *tci, char **tls_parameter_description, char **errstr)
+        mtls_cert_info_t *tci, char **tls_parameter_description, char **errstr)
 {
-    return tls_start(&session->tls, session->fd, tci,
+    return mtls_start(&session->mtls, session->fd, tci,
             tls_parameter_description, errstr);
 }
 #endif /* HAVE_TLS */
@@ -2314,6 +2317,72 @@ int pop3_auth_oauthbearer(pop3_session_t *session,
 }
 
 
+/*
+ * pop3_auth_xoauth2()
+ *
+ * Do POP3 authentication via AUTH XOAUTH2.
+ * The POP3 server must support POP3_CAP_AUTH_XOAUTH2
+ * Used error codes: POP3_EIO, POP3_EAUTHFAIL, POP3_EINVAL
+ */
+
+int pop3_auth_xoauth2(pop3_session_t *session,
+        const char *user, const char *token,
+        char **errmsg, char **errstr)
+{
+    int e;
+    char *oauth;
+    size_t oa_len;
+    char *b64;
+    size_t b64_len;
+
+    oa_len = 5 + /* "user=" */
+             strlen(user) +
+             13 + /* "^Aauth=Bearer " */
+             strlen(token) +
+             2 /* "^A^A" */;
+    oauth = xmalloc(oa_len + 1);
+    oa_len = snprintf(oauth, oa_len + 1,
+            "user=%s\001auth=Bearer %s\001\001", user, token);
+    b64_len = BASE64_LENGTH(oa_len) + 1;
+    b64 = xmalloc(b64_len);
+    base64_encode(oauth, oa_len, b64, b64_len);
+    e = pop3_send_cmd(session, errstr, "AUTH XOAUTH2 %s", b64);
+    free(oauth);
+    free(b64);
+    if (e != POP3_EOK)
+    {
+        return e;
+    }
+    if ((e = pop3_get_msg(session, 1, errstr)) != POP3_EOK)
+    {
+        return e;
+    }
+    if (session->buffer[0] == '+' && session->buffer[1] == 'O' && session->buffer[2] == 'K')
+    {
+        return POP3_EOK;
+    }
+    if (session->buffer[0] == '+' && session->buffer[1] == ' ')
+    {
+        if ((e = pop3_send_cmd(session, errstr, "")) != POP3_EOK)
+        {
+            return e;
+        }
+        if ((e = pop3_get_msg(session, 1, errstr)) != POP3_EOK)
+        {
+            return e;
+        }
+    }
+    if (!pop3_msg_ok(session->buffer))
+    {
+        *errmsg = xstrdup(session->buffer);
+        *errstr = xasprintf(_("authentication failed (method %s)"), "XOAUTH2");
+        return POP3_EAUTHFAIL;
+    }
+
+    return POP3_EOK;
+}
+
+
 /*
  * pop3_auth_plain()
  *
@@ -2634,7 +2703,9 @@ int pop3_server_supports_authmech(pop3_session_t *session, const char *mech)
             || ((session->cap.flags & POP3_CAP_AUTH_NTLM)
                 && strcmp(mech, "NTLM") == 0)
             || ((session->cap.flags & POP3_CAP_AUTH_OAUTHBEARER)
-                && strcmp(mech, "OAUTHBEARER") == 0));
+                && strcmp(mech, "OAUTHBEARER") == 0)
+            || ((session->cap.flags & POP3_CAP_AUTH_XOAUTH2)
+                && strcmp(mech, "XOAUTH2") == 0));
 }
 
 
@@ -2652,7 +2723,7 @@ int pop3_client_supports_authmech(const char *mech)
     Gsasl *ctx;
 
     if (strcmp(mech, "USER") == 0 || strcmp(mech, "APOP") == 0
-            || strcmp(mech, "OAUTHBEARER") == 0)
+            || strcmp(mech, "OAUTHBEARER") == 0 || strcmp(mech, "XOAUTH2") == 0)
     {
         supported = 1;
     }
@@ -2675,7 +2746,8 @@ int pop3_client_supports_authmech(const char *mech)
             || strcmp(mech, "PLAIN") == 0
             || strcmp(mech, "EXTERNAL") == 0
             || strcmp(mech, "LOGIN") == 0
-            || strcmp(mech, "OAUTHBEARER") == 0);
+            || strcmp(mech, "OAUTHBEARER") == 0
+            || strcmp(mech, "XOAUTH2") == 0);
 
 #endif /* not HAVE_LIBGSASL */
 }
@@ -2734,7 +2806,7 @@ int pop3_auth(pop3_session_t *session,
         /* Choose "best" authentication mechanism. */
         /* TODO: use gsasl_client_suggest_mechanism()? */
 #ifdef HAVE_TLS
-        if (tls_is_active(&session->tls))
+        if (mtls_is_active(&session->mtls))
         {
             if (gsasl_client_support_p(ctx, "PLAIN")
                     && (session->cap.flags & POP3_CAP_AUTH_PLAIN))
@@ -2789,7 +2861,7 @@ int pop3_auth(pop3_session_t *session,
     {
         gsasl_done(ctx);
 #ifdef HAVE_TLS
-        if (!tls_is_active(&session->tls))
+        if (!mtls_is_active(&session->mtls))
         {
 #endif /* HAVE_TLS */
             *errstr = xasprintf(_("cannot use a secure authentication method"));
@@ -2816,7 +2888,7 @@ int pop3_auth(pop3_session_t *session,
             return POP3_EUNAVAIL;
         }
         /* SCRAM-SHA-1, DIGEST-MD5, CRAM-MD5, PLAIN, LOGIN, NTLM, USER, APOP,
-         * OAUTHBEARER all need a password */
+         * OAUTHBEARER, XOAUTH2 all need a password */
         if (strcmp(auth_mech, "GSSAPI") != 0 && !password)
         {
             if (!password_callback
@@ -2832,7 +2904,7 @@ int pop3_auth(pop3_session_t *session,
         }
     }
 
-    /* USER, APOP, OAUTHBEARER are built-in, all other methods are provided by GNU SASL */
+    /* USER, APOP, OAUTHBEARER, XOAUTH2 are built-in, all other methods are provided by GNU SASL */
     if (strcmp(auth_mech, "USER") == 0)
     {
         gsasl_done(ctx);
@@ -2854,6 +2926,13 @@ int pop3_auth(pop3_session_t *session,
         free(callback_password);
         return e;
     }
+    else if (strcmp(auth_mech, "XOAUTH2") == 0)
+    {
+        gsasl_done(ctx);
+        e = pop3_auth_xoauth2(session, user, password, errmsg, errstr);
+        free(callback_password);
+        return e;
+    }
     else if ((error_code = gsasl_client_start(ctx, auth_mech, &sctx))
             != GSASL_OK)
     {
@@ -3019,7 +3098,7 @@ int pop3_auth(pop3_session_t *session,
     if (strcmp(auth_mech, "") == 0)
     {
 #ifdef HAVE_TLS
-        if (tls_is_active(&session->tls))
+        if (mtls_is_active(&session->mtls))
         {
             if (session->cap.flags & POP3_CAP_AUTH_PLAIN)
             {
@@ -3047,7 +3126,7 @@ int pop3_auth(pop3_session_t *session,
     if (strcmp(auth_mech, "") == 0)
     {
 #ifdef HAVE_TLS
-        if (!tls_is_active(&session->tls))
+        if (!mtls_is_active(&session->mtls))
         {
 #endif /* HAVE_TLS */
             *errstr = xasprintf(_("cannot use a secure authentication method"));
@@ -3065,7 +3144,7 @@ int pop3_auth(pop3_session_t *session,
     /* Check availability of required authentication data */
     if (strcmp(auth_mech, "EXTERNAL") != 0)
     {
-        /* CRAM-MD5, PLAIN, LOGIN, APOP, USER, OAUTHBEARER all need a user name and
+        /* CRAM-MD5, PLAIN, LOGIN, APOP, USER, OAUTHBEARER, XOAUTH2 all need a user name and
          * password */
         if (!user)
         {
@@ -3115,6 +3194,10 @@ int pop3_auth(pop3_session_t *session,
     {
         e = pop3_auth_oauthbearer(session, hostname, port, user, password, errmsg, errstr);
     }
+    else if (strcmp(auth_mech, "XOAUTH2") == 0)
+    {
+        e = pop3_auth_xoauth2(session, user, password, errmsg, errstr);
+    }
     else
     {
         *errstr = xasprintf(_("authentication method %s not supported"),
@@ -3193,9 +3276,9 @@ int pop3_quit(pop3_session_t *session, char **errmsg, char **errstr)
 void pop3_close(pop3_session_t *session)
 {
 #ifdef HAVE_TLS
-    if (tls_is_active(&session->tls))
+    if (mtls_is_active(&session->mtls))
     {
-        tls_close(&session->tls);
+        mtls_close(&session->mtls);
     }
 #endif /* HAVE_TLS */
     net_close_socket(session->fd);
