@@ -28,9 +28,8 @@
 #include <signal.h>
 
 #include "readbuf.h"
-#include "net.h"
 #ifdef HAVE_TLS
-# include "tls.h"
+# include "mtls.h"
 #endif /* HAVE_TLS */
 
 
@@ -70,16 +69,17 @@
 #define POP3_CAP_AUTH_LOGIN             (1 << 8)
 #define POP3_CAP_AUTH_NTLM              (1 << 9)
 #define POP3_CAP_AUTH_OAUTHBEARER       (1 << 10)
-#define POP3_CAP_CAPA                   (1 << 11)
-#define POP3_CAP_TOP                    (1 << 12)
-#define POP3_CAP_LOGIN_DELAY            (1 << 13)
-#define POP3_CAP_PIPELINING             (1 << 14)
-#define POP3_CAP_EXPIRE                 (1 << 15)
-#define POP3_CAP_UIDL                   (1 << 16)
-#define POP3_CAP_IMPLEMENTATION         (1 << 17)
-#define POP3_CAP_STLS                   (1 << 18)
-#define POP3_CAP_RESP_CODES             (1 << 19)
-#define POP3_CAP_AUTH_RESP_CODE         (1 << 20)
+#define POP3_CAP_AUTH_XOAUTH2           (1 << 11)
+#define POP3_CAP_CAPA                   (1 << 12)
+#define POP3_CAP_TOP                    (1 << 13)
+#define POP3_CAP_LOGIN_DELAY            (1 << 14)
+#define POP3_CAP_PIPELINING             (1 << 15)
+#define POP3_CAP_EXPIRE                 (1 << 16)
+#define POP3_CAP_UIDL                   (1 << 17)
+#define POP3_CAP_IMPLEMENTATION         (1 << 18)
+#define POP3_CAP_STLS                   (1 << 19)
+#define POP3_CAP_RESP_CODES             (1 << 20)
+#define POP3_CAP_AUTH_RESP_CODE         (1 << 21)
 
 /*
  * Buffer size for communication with the POP3 server. This restricts the length
@@ -177,7 +177,7 @@ typedef struct
     int count_newline_as_crlf;  /* does the server count newline as 2 chars? */
     int fd;                     /* the socket */
 #ifdef HAVE_TLS
-    tls_t tls;                  /* TLS descriptor */
+    mtls_t mtls;                /* TLS descriptor */
 #endif /* HAVE_TLS */
     char buffer[POP3_BUFSIZE];  /* input/output buffer */
     readbuf_t readbuf;          /* net input buffering */
@@ -320,7 +320,7 @@ int pop3_tls_stls(pop3_session_t *session, char **errmsg, char **errstr);
  */
 #ifdef HAVE_TLS
 int pop3_tls(pop3_session_t *session,
-        tls_cert_info_t *tci, char **tls_parameter_description, char **errstr);
+        mtls_cert_info_t *tci, char **tls_parameter_description, char **errstr);
 #endif /* HAVE_TLS */
 
 /*
